
function SetRGB(side, color, beat)
  xdrv.Set("track"..side.."_color_red", color.r, "beat", beat)
  xdrv.Set("track"..side.."_color_green", color.g, "beat", beat)
  xdrv.Set("track"..side.."_color_blue", color.b, "beat", beat)
end

function EaseRGB(side, startColor, endColor, beat, length, ease)
  if startColor.r ~= endColor.r then
    xdrv.Ease("track"..side.."_color_red", startColor.r, endColor.r, "beat", beat, "len", length, ease)
  end
  if startColor.g ~= endColor.g then
    xdrv.Ease("track"..side.."_color_green", startColor.g, endColor.g, "beat", beat, "len", length, ease)
  end
  if startColor.b ~= endColor.b then
    xdrv.Ease("track"..side.."_color_blue", startColor.b, endColor.b, "beat", beat, "len", length, ease)
  end
end

function LerpRGB(a, b, x)
  return {
    r = (a.r * (1-x)) + (b.r * x),
    g = (a.g * (1-x)) + (b.g * x),
    b = (a.b * (1-x)) + (b.b * x)
  }
end

function ToggleSidesOn(color, beat)
  SetRGB("", {r = 0, g = 0, b = 0}, beat)
  SetRGB("_left", color, beat)
  SetRGB("_right", color, beat)
end

function ToggleSidesOff(color, beat)
  SetRGB("_left", {r = 0, g = 0, b = 0}, beat)
  SetRGB("_right", {r = 0, g = 0, b = 0}, beat)
  SetRGB("", color, beat)
end

function HSV2RGB(h, s, v)
  if s == 0 then
    return {r = v, g = v, b = v}
  end
  local c = v * s
  local h_prime = h/60
  local x = c * ( 1 - math.abs((h_prime % 2) - 1) )
  
  local color_1 = nil
  if h_prime < 1 then color_1 = {r = c, g = x, b = 0}
  elseif h_prime < 2 then color_1 = {r = x, g = c, b = 0}
  elseif h_prime < 3 then color_1 = {r = 0, g = c, b = x}
  elseif h_prime < 4 then color_1 = {r = 0, g = x, b = c}
  elseif h_prime < 5 then color_1 = {r = x, g = 0, b = c}
  elseif h_prime < 6 then color_1 = {r = c, g = 0, b = x}
  else color_1 = {r = 0, g = 0, b = 0}
  end
  
  local m = v - c
  return {r = color_1.r + m, g = color_1.g + m, b = color_1.b + m}  
end

function EaseHueShift(side, h1, h2, s, v, beat, duration)
  local span = h2 - h1
  local span_segments = span/60
  local h1_prime = h1/60
  local h2_prime = h2/60
  
  local segment_duration = duration / span_segments
  local end_beat = beat + duration
  
  function _deg(hue_) return (hue_ % 6) * 60 end

  -- Get time from h1 to first integer segment
  local current_hue = h1_prime

  local start_overflow = h1_prime % 1
  if start_overflow > 0 then
    local start_duration = segment_duration * start_overflow
    current_hue = h1_prime - start_overflow + 1
    EaseRGB(side, HSV2RGB(h1, s, v), HSV2RGB(_deg(current_hue), s, v), beat, start_duration, "linear")
    beat = beat + start_duration
  end
  
  while current_hue + 1 < h2_prime do
    EaseRGB(side, HSV2RGB(_deg(current_hue), s, v), HSV2RGB(_deg(current_hue+1), s, v), beat, segment_duration, "linear")
    beat = beat + segment_duration
    current_hue = current_hue + 1
  end
  
  local remain = h2_prime - current_hue
  local end_duration = segment_duration * remain
  EaseRGB(side, HSV2RGB(_deg(current_hue), s, v), HSV2RGB(h2, s, v), beat, end_duration, "linear")
end

-- Setup colours

CHORUS = HSV2RGB(300, 1.00, 0.30)
BRIDGE = HSV2RGB(25, 1.00, 0.35)
VERSE = HSV2RGB(45, 1.00, 0.30)
GRENADE = HSV2RGB(45, 1.00, 0.40)
PULSE = HSV2RGB(280, 1.00, 0.35)
BASE = HSV2RGB(235, 1.00, 0.40)
DEFAULT = {r = 0.075, g = 0.075, b = 0.075}

-- Intro

EaseRGB("", DEFAULT, BASE, 32, 4, linear)
SetRGB("", VERSE, 69)

-- Beat 89: "GOLD GRENADE"

SetRGB("", GRENADE, 89)
xdrv.Set("camera_rotation_z", 20, "beat", 89)
xdrv.Set("camera_rotation_y", 10, "beat", 89)
xdrv.Set("camera_position_y", -1, "beat", 89)

xdrv.Set("camera_position_y", 0, "beat", 91)
xdrv.Set("camera_rotation_y", 0, "beat", 91)
xdrv.Set("camera_rotation_z", 0, "beat", 91)
SetRGB("", VERSE, 89)

-- Chorus colours

EaseRGB("", VERSE, CHORUS, 99, 2, "linear")
EaseRGB("", CHORUS, BRIDGE, 163, 2, "linear")
SetRGB("", VERSE, 197)
EaseRGB("", VERSE, CHORUS, 227, 2, "linear")

ToggleSidesOn(CHORUS, 259)
EaseRGB("_right", CHORUS, BRIDGE, 259, 2, "linear")
SetRGB("_right", CHORUS, 275)
SetRGB("_left", BRIDGE, 276)

-- Noises :)

xdrv.Set("track_color_alpha", 0, "beat", 291)
ToggleSidesOff(BASE, 291)
xdrv.Set("track_color_alpha", 1, "beat", 292)

-- Trill notes

TRILL_BASE = BASE
TRILL_INV = VERSE

function DoTrillColor(beat, duration)
  SetRGB("", TRILL_INV, beat)
  SetRGB("", TRILL_BASE, beat+duration)
  return DoTrillColor
end

DoTrillColor
  (305, 1)
  (307, 1)
  (321, 1)
  (323, 1)
  (337, 1)
  (339, 1)
  (353, 1)
  
  
-- Barks

function DoBark(side, beat)
  xdrv.Ease("track"..side.."_move_y", 0, 0.15, "beat", beat, "len", 0.5, "Bounce")
  return DoBark
end

DoBark
  ("left", 300.5)
  ("right", 301)
  ("right", 317)
  ("left", 332.5)
  ("right", 333)
  ("left", 349)
  ("right", 364.5)
  ("left", 365)
  ("right", 381)
  ("left", 396.5)
  ("right", 397)
  ("right", 413)
  
  

SetRGB("", CHORUS, 356)
-- subtle beat behind it

_prev = true

for beat = 357,418 do
  if
    beat == 370 or beat == 372 or
    beat == 386 or beat == 388 or
    beat == 402 or beat == 404 or
    beat == 418
  then
    EaseRGB("", BRIDGE, VERSE, beat - 0.150, 0.100, "InQuad")
  else
    EaseRGB("", CHORUS, PULSE, beat - 0.150, 0.100, "InQuad")
  end
  if
    beat == 369 or beat == 371 or
    beat == 385 or beat == 387 or
    beat == 401 or beat == 403 or
    beat == 417
  then
    SetRGB("", VERSE, beat)
    EaseRGB("", VERSE, BRIDGE, beat, 0.666, "OutSine")
    prev = false
  else
    if not prev then
      SetRGB("", PULSE, beat)
    end
    EaseRGB("", PULSE, CHORUS, beat, 0.666, "OutSine")
    prev = true
  end
end

SetRGB("", BASE, 419)
SetRGB("", CHORUS, 419.250)
SetRGB("", BASE, 419.500)
SetRGB("", CHORUS, 419.750)
EaseRGB("", VERSE, CHORUS, 420, 6, "OutSine")

EaseHueShift("", 300, 235+1440, 1.00, 0.30, 484, 63)

SetRGB("", CHORUS, 547)
SetRGB("", BASE, 547.250)
SetRGB("", CHORUS, 547.500)
SetRGB("", BASE, 547.750)

EaseRGB("", BASE, DEFAULT, 548, 4, "linear")



-- Tail end of the song fades out lights, ui, screen
-- 582.000 = lights, event
-- 582.750 = ui
-- 583.500 = screen
xdrv.Set("black_bar_top_position", 1, "beat", 583.5)

